@@ -13,11 +13,12 @@
 # limitations under the License.
 
 __all__ = ['BatchNorm', 'BatchNorm0D', 'BatchNorm1D', 'BatchNorm2D',
-           'Conv2D', 'ConvTranspose2D', 'Dropout', 'Linear',
-           'MovingAverage', 'ExponentialMovingAverage', 'Sequential',
+           'Conv2D', 'ConvTranspose2D', 'Dropout', 'ExponentialMovingAverage',
+           'GroupNorm', 'GroupNorm0D', 'GroupNorm1D', 'GroupNorm2D',
+           'Linear', 'MovingAverage', 'Sequential',
            'SyncedBatchNorm', 'SyncedBatchNorm0D', 'SyncedBatchNorm1D', 'SyncedBatchNorm2D']
 
-from typing import Callable, Iterable, Tuple, Optional, Union, List
+from typing import Callable, Iterable, Tuple, Optional, Union, List, Dict
 
 from jax import numpy as jn, random as jr, lax
 
@@ -26,6 +27,7 @@ from objax.constants import ConvPadding
 from objax.module import ModuleList, Module
 from objax.nn.init import kaiming_normal, xavier_normal
 from objax.typing import JaxArray, ConvPaddingInt
+from objax.util import class_name
 from objax.variable import TrainVar, StateVar
 
 
@@ -69,7 +71,7 @@ class BatchNorm(Module):
         """
         if training:
             m = x.mean(self.redux, keepdims=True)
-            v = (x ** 2).mean(self.redux, keepdims=True) - m ** 2
+            v = ((x - m) ** 2).mean(self.redux, keepdims=True)  # Note: x^2 - m^2 is not numerically stable.
             self.running_mean.value += (1 - self.momentum) * (m - self.running_mean.value)
             self.running_var.value += (1 - self.momentum) * (v - self.running_var.value)
         else:
@@ -77,6 +79,11 @@ class BatchNorm(Module):
         y = self.gamma.value * (x - m) * functional.rsqrt(v + self.eps) + self.beta.value
         return y
 
+    def __repr__(self):
+        args = dict(dims=self.beta.value.shape, redux=self.redux, momentum=self.momentum, eps=self.eps)
+        args = ', '.join(f'{x}={y}' for x, y in args.items())
+        return f'{class_name(self)}({args})'
+
 
 class BatchNorm0D(BatchNorm):
     """Applies a 0D batch normalization on a 2D-input batch of shape (N,C).
@@ -96,6 +103,9 @@ class BatchNorm0D(BatchNorm):
         """
         super().__init__((1, nin), (0,), momentum, eps)
 
+    def __repr__(self):
+        return f'{class_name(self)}(nin={self.beta.value.shape[1]}, momentum={self.momentum}, eps={self.eps})'
+
 
 class BatchNorm1D(BatchNorm):
     """Applies a 1D batch normalization on a 3D-input batch of shape (N,C,L).
@@ -115,6 +125,9 @@ class BatchNorm1D(BatchNorm):
         """
         super().__init__((1, nin, 1), (0, 2), momentum, eps)
 
+    def __repr__(self):
+        return f'{class_name(self)}(nin={self.beta.value.shape[1]}, momentum={self.momentum}, eps={self.eps})'
+
 
 class BatchNorm2D(BatchNorm):
     """Applies a 2D batch normalization on a 4D-input batch of shape (N,C,H,W).
@@ -134,6 +147,9 @@ class BatchNorm2D(BatchNorm):
         """
         super().__init__((1, nin, 1, 1), (0, 2, 3), momentum, eps)
 
+    def __repr__(self):
+        return f'{class_name(self)}(nin={self.beta.value.shape[1]}, momentum={self.momentum}, eps={self.eps})'
+
 
 class Conv2D(Module):
     """Applies a 2D convolution on a 4D-input batch of shape (N,C,H,W)."""
@@ -166,27 +182,35 @@ class Conv2D(Module):
         super().__init__()
         assert nin % groups == 0, 'nin should be divisible by groups'
         assert nout % groups == 0, 'nout should be divisible by groups'
-        self.nin = nin
         self.b = TrainVar(jn.zeros((nout, 1, 1))) if use_bias else None
         self.w = TrainVar(w_init((*util.to_tuple(k, 2), nin // groups, nout)))  # HWIO
         self.padding = util.to_padding(padding, 2)
         self.strides = util.to_tuple(strides, 2)
         self.dilations = util.to_tuple(dilations, 2)
         self.groups = groups
+        self.w_init = w_init
 
     def __call__(self, x: JaxArray) -> JaxArray:
         """Returns the results of applying the convolution to input x."""
-        assert x.shape[1] == self.nin, (f'Attempting to convolve an input with {x.shape[1]} input channels '
-                                        f'when the convolution expects {self.nin} channels. For reference, '
-                                        f'self.w.value.shape={self.w.value.shape} and x.shape={x.shape}.')
+        nin = self.w.value.shape[2] * self.groups
+        assert x.shape[1] == nin, (f'Attempting to convolve an input with {x.shape[1]} input channels '
+                                   f'when the convolution expects {nin} channels. For reference, '
+                                   f'self.w.value.shape={self.w.value.shape} and x.shape={x.shape}.')
         y = lax.conv_general_dilated(x, self.w.value, self.strides, self.padding,
                                      rhs_dilation=self.dilations,
                                      feature_group_count=self.groups,
                                      dimension_numbers=('NCHW', 'HWIO', 'NCHW'))
-        if self.b:
+        if self.b is not None:
             y += self.b.value
         return y
 
+    def __repr__(self):
+        args = dict(nin=self.w.value.shape[2] * self.groups, nout=self.w.value.shape[3], k=self.w.value.shape[:2],
+                    strides=self.strides, dilations=self.dilations, groups=self.groups, padding=self.padding,
+                    use_bias=self.b is not None)
+        args = ', '.join(f'{k}={repr(v)}' for k, v in args.items())
+        return f'{class_name(self)}({args}, w_init={util.repr_function(self.w_init)})'
+
 
 class ConvTranspose2D(Conv2D):
     """Applies a 2D transposed convolution on a 4D-input batch of shape (N,C,H,W).
@@ -223,19 +247,26 @@ class ConvTranspose2D(Conv2D):
             use_bias: if True then convolution will have bias term.
             w_init: initializer for convolution kernel (a function that takes in a HWIO shape and returns a 4D matrix).
         """
-        super().__init__(nin=nout, nout=nin, k=k, strides=strides, padding=padding, use_bias=False, w_init=w_init)
+        super().__init__(nin=nout, nout=nin, k=k, strides=strides, dilations=dilations, padding=padding,
+                         use_bias=False, w_init=w_init)
         self.b = TrainVar(jn.zeros((nout, 1, 1))) if use_bias else None
-        self.dilations = util.to_tuple(dilations, 2)
 
     def __call__(self, x: JaxArray) -> JaxArray:
         """Returns the results of applying the transposed convolution to input x."""
         y = lax.conv_transpose(x, self.w.value, self.strides, self.padding,
                                rhs_dilation=self.dilations,
                                dimension_numbers=('NCHW', 'HWIO', 'NCHW'), transpose_kernel=True)
-        if self.b:
+        if self.b is not None:
             y += self.b.value
         return y
 
+    def __repr__(self):
+        args = dict(nin=self.w.value.shape[3], nout=self.w.value.shape[2], k=self.w.value.shape[:2],
+                    strides=self.strides, dilations=self.dilations, padding=self.padding,
+                    use_bias=self.b is not None)
+        args = ', '.join(f'{k}={repr(v)}' for k, v in args.items())
+        return f'{class_name(self)}({args}, w_init={util.repr_function(self.w_init)})'
+
 
 class Dropout(Module):
     """In the training phase, a dropout layer zeroes some elements of the input tensor with probability 1-keep and
@@ -268,48 +299,8 @@ class Dropout(Module):
         keep_mask = jr.bernoulli(self.keygen(), keep, x.shape)
         return jn.where(keep_mask, x / keep, 0)
 
-
-class Linear(Module):
-    """Applies a linear transformation on an input batch."""
-
-    def __init__(self, nin: int, nout: int, use_bias: bool = True, w_init: Callable = xavier_normal):
-        """Creates a Linear module instance.
-
-        Args:
-            nin: number of channels of the input tensor.
-            nout: number of channels of the output tensor.
-            use_bias: if True then linear layer will have bias term.
-            w_init: weight initializer for linear layer (a function that takes in a IO shape and returns a 2D matrix).
-        """
-        super().__init__()
-        self.b = TrainVar(jn.zeros(nout)) if use_bias else None
-        self.w = TrainVar(w_init((nin, nout)))
-
-    def __call__(self, x: JaxArray) -> JaxArray:
-        """Returns the results of applying the linear transformation to input x."""
-        y = jn.dot(x, self.w.value)
-        if self.b:
-            y += self.b.value
-        return y
-
-
-class MovingAverage(Module):
-    """Computes moving average of an input batch."""
-
-    def __init__(self, shape: Tuple[int, ...], buffer_size: int, init_value: float = 0):
-        """Creates a MovingAverage module instance.
-
-        Args:
-            shape: shape of the input tensor.
-            buffer_size: buffer size for moving average.
-            init_value: initial value for moving average buffer.
-        """
-        self.buffer = StateVar(jn.zeros((buffer_size,) + shape) + init_value)
-
-    def __call__(self, x: JaxArray) -> JaxArray:
-        """Update the statistics using x and return the moving average."""
-        self.buffer.value = jn.concatenate([self.buffer.value[1:], x[None]])
-        return self.buffer.value.mean(0)
+    def __repr__(self):
+        return f'{class_name(self)}(keep={self.keep})'
 
 
 class ExponentialMovingAverage(Module):
@@ -324,6 +315,7 @@ class ExponentialMovingAverage(Module):
             init_value: initial value for exponential moving average.
         """
         self.momentum = momentum
+        self.init_value = init_value
         self.avg = StateVar(jn.zeros(shape) + init_value)
 
     def __call__(self, x: JaxArray) -> JaxArray:
@@ -331,19 +323,181 @@ class ExponentialMovingAverage(Module):
         self.avg.value += (self.avg.value - x) * (self.momentum - 1)
         return self.avg.value
 
+    def __repr__(self):
+        s = self.avg.value.shape
+        return f'{class_name(self)}(shape={s}, momentum={self.momentum}, init_value={self.init_value})'
+
+
+class GroupNorm(Module):
+    """Applies a group normalization to input tensor.
+
+    The module follows the operation described the paper
+    `Group Normalization <https://arxiv.org/abs/1803.08494>`_.
+    """
+
+    def __init__(self, nin: int, rank: int, groups: int = 32, eps: float = 1e-5):
+        """Creates a GroupNorm module instance.
+
+        Args:
+            nin: number of input channels.
+            rank: rank of the input tensor.
+            groups: number of normalization groups.
+            eps: small value which is used for numerical stability.
+        """
+        groups = min(groups, nin)
+        assert nin % groups == 0, 'nin should be divisible by groups'
+
+        super(GroupNorm, self).__init__()
+        self.nin = nin
+        self.groups = groups
+        self.eps = eps
+        self.redux = tuple(range(2, rank + 1))
+        var_shape = (1, nin) + (1,) * (rank - 2)
+        self.gamma = TrainVar(jn.ones(var_shape))
+        self.beta = TrainVar(jn.zeros(var_shape))
+
+    def __call__(self, x: JaxArray, training: bool = True) -> JaxArray:
+        """Returns the results of applying group normalization to input x."""
+        # This method has unused training argument, so group norm can be used as a drop-in replacement of batch norm.
+        del training
+        group_shape = ((-1, self.groups, self.nin // self.groups) + x.shape[2:])
+        x = x.reshape(group_shape)
+        mean = x.mean(axis=self.redux, keepdims=True)
+        var = x.var(axis=self.redux, keepdims=True)
+        x = (x - mean) * functional.rsqrt(var + self.eps)
+        x = x.reshape((-1, self.nin,) + group_shape[3:])
+        x = x * self.gamma.value + self.beta.value
+        return x
+
+    def __repr__(self):
+        args = f'nin={self.nin}, rank={len(self.gamma.shape)}, groups={self.groups}, eps={self.eps}'
+        return f'{class_name(self)}({args})'
+
+
+class GroupNorm0D(GroupNorm):
+    """Applies a 0D group normalization on a input batch of shape (N,C)."""
+
+    def __init__(self, nin: int, groups: int = 32, eps: float = 1e-5):
+        """Creates a GroupNorm2D module instance.
+
+        Args:
+            nin: number of input channels.
+            groups: number of normalization groups.
+            eps: small value which is used for numerical stability.
+        """
+        super().__init__(nin, rank=2, groups=groups, eps=eps)
+
+    def __repr__(self):
+        return f'{class_name(self)}(nin={self.nin}, groups={self.groups}, eps={self.eps})'
+
+
+class GroupNorm1D(GroupNorm):
+    """Applies a 1D group normalization on a input batch of shape (N,C,L)."""
+
+    def __init__(self, nin: int, groups: int = 32, eps: float = 1e-5):
+        """Creates a GroupNorm2D module instance.
+
+        Args:
+            nin: number of input channels.
+            groups: number of normalization groups.
+            eps: small value which is used for numerical stability.
+        """
+        super().__init__(nin, rank=3, groups=groups, eps=eps)
+
+    def __repr__(self):
+        return f'{class_name(self)}(nin={self.nin}, groups={self.groups}, eps={self.eps})'
+
+
+class GroupNorm2D(GroupNorm):
+    """Applies a 2D group normalization on a input batch of shape (N,C,H,W)."""
+
+    def __init__(self, nin: int, groups: int = 32, eps: float = 1e-5):
+        """Creates a GroupNorm2D module instance.
+
+        Args:
+            nin: number of input channels.
+            groups: number of normalization groups.
+            eps: small value which is used for numerical stability.
+        """
+        super().__init__(nin, rank=4, groups=groups, eps=eps)
+
+    def __repr__(self):
+        return f'{class_name(self)}(nin={self.nin}, groups={self.groups}, eps={self.eps})'
+
+
+class Linear(Module):
+    """Applies a linear transformation on an input batch."""
+
+    def __init__(self, nin: int, nout: int, use_bias: bool = True, w_init: Callable = xavier_normal):
+        """Creates a Linear module instance.
+
+        Args:
+            nin: number of channels of the input tensor.
+            nout: number of channels of the output tensor.
+            use_bias: if True then linear layer will have bias term.
+            w_init: weight initializer for linear layer (a function that takes in a IO shape and returns a 2D matrix).
+        """
+        super().__init__()
+        self.w_init = w_init
+        self.b = TrainVar(jn.zeros(nout)) if use_bias else None
+        self.w = TrainVar(w_init((nin, nout)))
+
+    def __call__(self, x: JaxArray) -> JaxArray:
+        """Returns the results of applying the linear transformation to input x."""
+        y = jn.dot(x, self.w.value)
+        if self.b is not None:
+            y += self.b.value
+        return y
+
+    def __repr__(self):
+        s = self.w.value.shape
+        args = f'nin={s[0]}, nout={s[1]}, use_bias={self.b is not None}, w_init={util.repr_function(self.w_init)}'
+        return f'{class_name(self)}({args})'
+
+
+class MovingAverage(Module):
+    """Computes moving average of an input batch."""
+
+    def __init__(self, shape: Tuple[int, ...], buffer_size: int, init_value: float = 0):
+        """Creates a MovingAverage module instance.
+
+        Args:
+            shape: shape of the input tensor.
+            buffer_size: buffer size for moving average.
+            init_value: initial value for moving average buffer.
+        """
+        self.init_value = init_value
+        self.buffer = StateVar(jn.zeros((buffer_size,) + shape) + init_value)
+
+    def __call__(self, x: JaxArray) -> JaxArray:
+        """Update the statistics using x and return the moving average."""
+        self.buffer.value = jn.concatenate([self.buffer.value[1:], x[None]])
+        return self.buffer.value.mean(0)
+
+    def __repr__(self):
+        s = self.buffer.value.shape
+        return f'{class_name(self)}(shape={s[1:]}, buffer_size={s[0]}, init_value={self.init_value})'
+
 
 class Sequential(ModuleList):
     """Executes modules in the order they were passed to the constructor."""
 
+    @staticmethod
+    def run_layer(layer: int, f: Callable, args: List, kwargs: Dict):
+        try:
+            return f(*args, **util.local_kwargs(kwargs, f))
+        except Exception as e:
+            raise type(e)(f'Sequential layer[{layer}] {f} {e}') from e
+
     def __call__(self, *args, **kwargs) -> Union[JaxArray, List[JaxArray]]:
         """Execute the sequence of operations contained on ``*args`` and ``**kwargs`` and return result."""
         if not self:
             return args if len(args) > 1 else args[0]
-        for f in self[:-1]:
-            args = f(*args, **util.local_kwargs(kwargs, f))
+        for i, f in enumerate(self[:-1]):
+            args = self.run_layer(i, f, args, kwargs)
             if not isinstance(args, tuple):
                 args = (args,)
-        return self[-1](*args, **util.local_kwargs(kwargs, self[-1]))
+        return self.run_layer(len(self) - 1, self[-1], args, kwargs)
 
     def __getitem__(self, key: Union[int, slice]):
         value = list.__getitem__(self, key)
@@ -358,7 +512,7 @@ class SyncedBatchNorm(BatchNorm):
     def __call__(self, x: JaxArray, training: bool, batch_norm_update: bool = True) -> JaxArray:
         if training:
             m = functional.parallel.pmean(x.mean(self.redux, keepdims=True))
-            v = functional.parallel.pmean((x ** 2).mean(self.redux, keepdims=True) - m ** 2)
+            v = functional.parallel.pmean(((x - m) ** 2).mean(self.redux, keepdims=True))
             if batch_norm_update:
                 self.running_mean.value += (1 - self.momentum) * (m - self.running_mean.value)
                 self.running_var.value += (1 - self.momentum) * (v - self.running_var.value)
@@ -385,6 +539,9 @@ class SyncedBatchNorm0D(SyncedBatchNorm):
         """
         super().__init__((1, nin), (0,), momentum, eps)
 
+    def __repr__(self):
+        return f'{class_name(self)}(nin={self.beta.value.shape[1]}, momentum={self.momentum}, eps={self.eps})'
+
 
 class SyncedBatchNorm1D(SyncedBatchNorm):
     """Applies a 1D synchronized batch normalization on a 3D-input batch of shape (N,C,L).
@@ -403,6 +560,9 @@ class SyncedBatchNorm1D(SyncedBatchNorm):
         """
         super().__init__((1, nin, 1), (0, 2), momentum, eps)
 
+    def __repr__(self):
+        return f'{class_name(self)}(nin={self.beta.value.shape[1]}, momentum={self.momentum}, eps={self.eps})'
+
 
 class SyncedBatchNorm2D(SyncedBatchNorm):
     """Applies a 2D synchronized batch normalization on a 4D-input batch of shape (N,C,H,W).
@@ -420,3 +580,6 @@ class SyncedBatchNorm2D(SyncedBatchNorm):
             eps: small value which is used for numerical stability.
         """
         super().__init__((1, nin, 1, 1), (0, 2, 3), momentum, eps)
+
+    def __repr__(self):
+        return f'{class_name(self)}(nin={self.beta.value.shape[1]}, momentum={self.momentum}, eps={self.eps})'
