@@ -14,5 +14,7 @@
 
 from .adam import *
 from .ema import *
+from .lars import *
 from .momentum import *
 from .sgd import *
+from . import scheduler
\ No newline at end of file
