@@ -15,3 +15,5 @@
 from . import image
 from . import check
 from .util import *
+from .objax2tf import Objax2Tf
+from .tracing import find_used_variables
\ No newline at end of file
